@@ -42,6 +42,8 @@
 extern "C" {
 #endif
 
+#include <alsa/asoundlib.h>
+
 /**
  *  \defgroup ucm Use Case Interface
  *  The ALSA Use Case manager interface.
@@ -84,7 +86,7 @@ extern "C" {
  *  + Get the TQ parameter for each use case verb, use case device and
  *     modifier.
  *  + Get the ALSA master playback and capture volume/switch kcontrols
- *     for each use case.
+ *     or mixer elements for each use case.
  */
 
 
@@ -112,10 +114,22 @@ extern "C" {
  *
  * Physical system devices the render and capture audio. Devices can be OR'ed
  * together to support audio on simultaneous devices.
+ *
+ * If multiple devices with the same name exists, the number suffixes should
+ * be added to these names like HDMI1,HDMI2,HDMI3 etc. No number gaps are
+ * allowed. The names with numbers must be continuous.
+ *
+ * If EnableSequence/DisableSequence controls independent paths in the hardware
+ * it is also recommended to split playback and capture UCM devices and use
+ * the number suffixes. Example use case: Use the integrated microphone
+ * in the laptop instead the microphone in headphones.
+ *
+ * The preference of the devices is determined by the priority value.
  */
 #define SND_USE_CASE_DEV_NONE		"None"		/**< None Device */
 #define SND_USE_CASE_DEV_SPEAKER	"Speaker"	/**< Speaker Device */
 #define SND_USE_CASE_DEV_LINE		"Line"		/**< Line Device */
+#define SND_USE_CASE_DEV_MIC            "Mic"           /**< Microphone Device */
 #define SND_USE_CASE_DEV_HEADPHONES	"Headphones"	/**< Headphones Device */
 #define SND_USE_CASE_DEV_HEADSET	"Headset"	/**< Headset Device */
 #define SND_USE_CASE_DEV_HANDSET	"Handset"	/**< Handset Device */
@@ -204,6 +218,7 @@ int snd_use_case_free_list(const char *list[], int items);
  *   - _enadevs			- get list of enabled devices
  *   - _enamods			- get list of enabled modifiers
  *
+ *   - _identifiers/{modifier}|{device}[/{verb}]     - list of value identifiers
  *   - _supporteddevs/{modifier}|{device}[/{verb}]   - list of supported devices
  *   - _conflictingdevs/{modifier}|{device}[/{verb}] - list of conflicting devices
  *
@@ -259,6 +274,10 @@ int snd_use_case_get_list(snd_use_case_mgr_t *uc_mgr,
  * Recommended names for values:
  *   - TQ
  *      - Tone Quality
+ *   - Priority
+ *      - priority value (1-10000), higher value means higher priority
+ *      - valid only for verbs
+ *      - for devices - PlaybackPriority and CapturePriority
  *   - PlaybackPCM
  *      - full PCM playback device name
  *   - PlaybackPCMIsDummy
@@ -280,11 +299,13 @@ int snd_use_case_get_list(snd_use_case_mgr_t *uc_mgr,
  *   - PlaybackCTL
  *      - playback control device name
  *   - PlaybackVolume
- *      - playback control volume ID string
+ *      - playback control volume identifier string
  *	- can be parsed using snd_use_case_parse_ctl_elem_id()
  *   - PlaybackSwitch
- *      - playback control switch ID string
+ *      - playback control switch identifier string
  *	- can be parsed using snd_use_case_parse_ctl_elem_id()
+ *   - PlaybackPriority
+ *      - priority value (1-10000), higher value means higher priority
  *   - CaptureRate
  *      - capture device sample rate
  *   - CaptureChannels
@@ -292,42 +313,63 @@ int snd_use_case_get_list(snd_use_case_mgr_t *uc_mgr,
  *   - CaptureCTL
  *      - capture control device name
  *   - CaptureVolume
- *      - capture control volume ID string
+ *      - capture control volume identifier string
  *	- can be parsed using snd_use_case_parse_ctl_elem_id()
  *   - CaptureSwitch
- *      - capture control switch ID string
+ *      - capture control switch identifier string
  *	- can be parsed using snd_use_case_parse_ctl_elem_id()
+ *   - CapturePriority
+ *      - priority value (1-10000), higher value means higher priority
  *   - PlaybackMixer
  *      - name of playback mixer
- *   - PlaybackMixerID
- *      - mixer playback ID
+ *   - PlaybackMixerElem
+ *      - mixer element playback identifier
  *	- can be parsed using snd_use_case_parse_selem_id()
+ *   - PlaybackMasterElem
+ *      - mixer element playback identifier for the master control
+ *	- can be parsed using snd_use_case_parse_selem_id()
+ *   - PlaybackMasterType
+ *      - type of the master volume control
+ *      - Valid values: "soft" (software attenuation)
  *   - CaptureMixer
  *      - name of capture mixer
- *   - CaptureMixerID
+ *   - CaptureMixerElem
+ *      - mixer element capture identifier
  *	- can be parsed using snd_use_case_parse_selem_id()
- *      - mixer capture ID
+ *   - CaptureMasterElem
+ *      - mixer element playback identifier for the master control
+ *	- can be parsed using snd_use_case_parse_selem_id()
+ *   - CaptureMasterType
+ *      - type of the master volume control
+ *      - Valid values: "soft" (software attenuation)
  *   - EDIDFile
  *      - Path to EDID file for HDMI devices
- *   - JackControl, JackDev, JackHWMute
- *      - Jack information for a device. The jack status can be reported via
- *        a kcontrol and/or via an input device. **JackControl** is the
- *        kcontrol name of the jack, and **JackDev** is the input device id of
- *        the jack (if the full input device path is /dev/input/by-id/foo, the
- *        JackDev value should be "foo"). UCM configuration files should
- *        contain both JackControl and JackDev when possible, because
- *        applications are likely to support only one or the other.
- *
- *        If **JackHWMute** is set, it indicates that when the jack is plugged
- *        in, the hardware automatically mutes some other device(s). The
- *        JackHWMute value is a space-separated list of device names (this
- *        isn't compatible with device names with spaces in them, so don't use
- *        such device names!). Note that JackHWMute should be used only when
- *        the hardware enforces the automatic muting. If the hardware doesn't
- *        enforce any muting, it may still be tempting to set JackHWMute to
- *        trick upper software layers to e.g. automatically mute speakers when
- *        headphones are plugged in, but that's application policy
- *        configuration that doesn't belong to UCM configuration files.
+ *   - JackCTL
+ *      - jack control device name
+ *   - JackControl
+ *      - jack control identificator
+ *      - can be parsed using snd_use_case_parse_ctl_elem_id()
+ *      - UCM configuration files should contain both JackControl and JackDev
+ *        when possible, because applications are likely to support only one
+ *        or the other
+ *   - JackDev
+ *      - the input device id of the jack (if the full input device path is
+ *        /dev/input/by-id/foo, the JackDev value should be "foo")
+ *      - UCM configuration files should contain both JackControl and JackDev
+ *        when possible, because applications are likely to support only one
+ *        or the other
+ *   - JackHWMute
+ *	  If this value is set, it indicates that when the jack is plugged
+ *	  in, the hardware automatically mutes some other device(s). The
+ *	  value is a space-separated list of device names. If the device
+ *	  name contains space, it must be enclosed to ' or ", e.g.:
+ *		JackHWMute "'Dock Headphone' Headphone"
+ *        Note that JackHWMute should be used only when the hardware enforces
+ *        the automatic muting. If the hardware doesn't enforce any muting, it
+ *        may still be tempting to set JackHWMute to trick upper software layers
+ *        to e.g. automatically mute speakers when headphones are plugged in,
+ *        but that's application policy configuration that doesn't belong
+ *        to UCM configuration files.
  *   - MinBufferLevel
  *	- This is used on platform where reported buffer level is not accurate.
  *	  E.g. "512", which holds 512 samples in device buffer. Note: this will
