@@ -22,7 +22,7 @@
  *
  *   You should have received a copy of the GNU Lesser General Public
  *   License along with this library; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -400,8 +400,8 @@ static void MIX_AREAS_24(unsigned int size,
 		"\tmovzwl (%%esi), %%ecx\n"
 		"\tmovl (%%ebx), %%edx\n"
 		"\tsall $16, %%eax\n"
+		"\torl %%eax, %%ecx\n"
 		"\t" LOCK_PREFIX "btsw $0, (%%edi)\n"
-		"\tleal (%%ecx,%%eax,1), %%ecx\n"
 		"\tjc 2f\n"
 		"\t" XSUB " %%edx, %%ecx\n"
 		"2:"
