@@ -41,9 +41,10 @@
  * misc
  */
 
-static int get_value1(char **value, struct list_head *value_list,
-                      const char *identifier);
-static int get_value3(char **value,
+static int get_value1(snd_use_case_mgr_t *uc_mgr, char **value,
+                      struct list_head *value_list, const char *identifier);
+static int get_value3(snd_use_case_mgr_t *uc_mgr,
+                      char **value,
 		      const char *identifier,
 		      struct list_head *value_list1,
 		      struct list_head *value_list2,
@@ -60,11 +61,13 @@ static int check_identifier(const char *identifier, const char *prefix)
 {
 	int len;
 
-	if (strcmp(identifier, prefix) == 0)
-		return 1;
 	len = strlen(prefix);
-	if (memcmp(identifier, prefix, len) == 0 && identifier[len] == '/')
+	if (strncmp(identifier, prefix, len) != 0)
+		return 0;
+
+	if (identifier[len] == 0 || identifier[len] == '/')
 		return 1;
+
 	return 0;
 }
 
@@ -138,37 +141,6 @@ int snd_use_case_free_list(const char *list[], int items)
 	return 0;
 }
 
-static int open_ctl(snd_use_case_mgr_t *uc_mgr,
-		    snd_ctl_t **ctl,
-		    const char *ctl_dev)
-{
-	int err;
-
-	/* FIXME: add a list of ctl devices to uc_mgr structure and
-           cache accesses for multiple opened ctl devices */
-	if (uc_mgr->ctl_dev != NULL && strcmp(ctl_dev, uc_mgr->ctl_dev) == 0) {
-		*ctl = uc_mgr->ctl;
-		return 0;
-	}
-	if (uc_mgr->ctl_dev) {
-		free(uc_mgr->ctl_dev);
-		uc_mgr->ctl_dev = NULL;
-		snd_ctl_close(uc_mgr->ctl);
-		uc_mgr->ctl = NULL;
-	
-	}
-	err = snd_ctl_open(ctl, ctl_dev, 0);
-	if (err < 0)
-		return err;
-	uc_mgr->ctl_dev = strdup(ctl_dev);
-	if (uc_mgr->ctl_dev == NULL) {
-		snd_ctl_close(*ctl);
-		return -ENOMEM;
-	}
-	uc_mgr->ctl = *ctl;
-	return 0;
-}
-
 static int read_tlv_file(unsigned int **res,
 			 const char *filepath)
 {
@@ -390,7 +362,7 @@ static int execute_sequence(snd_use_case_mgr_t *uc_mgr,
 				char *playback_ctl = NULL;
 				char *capture_ctl = NULL;
 
-				err = get_value3(&playback_ctl, "PlaybackCTL",
+				err = get_value3(uc_mgr, &playback_ctl, "PlaybackCTL",
 						 value_list1,
 						 value_list2,
 						 value_list3);
@@ -398,7 +370,7 @@ static int execute_sequence(snd_use_case_mgr_t *uc_mgr,
 					uc_error("cdev is not defined!");
 					return err;
 				}
-				err = get_value3(&capture_ctl, "CaptureCTL",
+				err = get_value3(uc_mgr, &capture_ctl, "CaptureCTL",
 						 value_list1,
 						 value_list2,
 						 value_list3);
@@ -417,17 +389,18 @@ static int execute_sequence(snd_use_case_mgr_t *uc_mgr,
 				    strcmp(playback_ctl, capture_ctl) != 0) {
 					free(playback_ctl);
 					free(capture_ctl);
-					uc_error("cdev is not defined!");
+					uc_error("cdev is not equal for playback and capture!");
 					return -EINVAL;
 				}
 				if (playback_ctl != NULL) {
 					cdev = playback_ctl;
 					free(capture_ctl);
-				} else
+				} else {
 					cdev = capture_ctl;
+				}
 			}
 			if (ctl == NULL) {
-				err = open_ctl(uc_mgr, &ctl, cdev);
+				err = uc_mgr_open_ctl(uc_mgr, &ctl, cdev);
 				if (err < 0) {
 					uc_error("unable to open ctl device '%s'", cdev);
 					goto __fail;
@@ -435,7 +408,7 @@ static int execute_sequence(snd_use_case_mgr_t *uc_mgr,
 			}
 			err = execute_cset(ctl, s->data.cset, s->type);
 			if (err < 0) {
-				uc_error("unable to execute cset '%s'\n", s->data.cset);
+				uc_error("unable to execute cset '%s'", s->data.cset);
 				goto __fail;
 			}
 			break;
@@ -518,6 +491,45 @@ static int execute_component_seq(snd_use_case_mgr_t *uc_mgr,
 	return err;
 }
 
+static int add_auto_value(snd_use_case_mgr_t *uc_mgr, const char *key, char *value)
+{
+	char *s;
+	int err;
+
+	err = get_value1(uc_mgr, &value, &uc_mgr->value_list, key);
+	if (err == -ENOENT) {
+		s = strdup(value);
+		if (s == NULL)
+			return -ENOMEM;
+		return uc_mgr_add_value(&uc_mgr->value_list, key, s);
+	} else if (err < 0) {
+		return err;
+	}
+	free(value);
+	return 0;
+}
+
+static int add_auto_values(snd_use_case_mgr_t *uc_mgr)
+{
+	struct ctl_list *ctl_list;
+	const char *id;
+	char buf[40];
+	int err;
+
+	ctl_list = uc_mgr_get_one_ctl(uc_mgr);
+	if (ctl_list) {
+		id = snd_ctl_card_info_get_id(ctl_list->ctl_info);
+		snprintf(buf, sizeof(buf), "hw:%s", id);
+		err = add_auto_value(uc_mgr, "PlaybackCTL", buf);
+		if (err < 0)
+			return err;
+		err = add_auto_value(uc_mgr, "CaptureCTL", buf);
+		if (err < 0)
+			return err;
+	}
+	return 0;
+}
+
 /**
  * \brief Import master config and execute the default sequence
  * \param uc_mgr Use case manager
@@ -528,6 +540,9 @@ static int import_master_config(snd_use_case_mgr_t *uc_mgr)
 	int err;
 	
 	err = uc_mgr_import_master_config(uc_mgr);
+	if (err < 0)
+		return err;
+	err = add_auto_values(uc_mgr);
 	if (err < 0)
 		return err;
 	err = execute_sequence(uc_mgr, &uc_mgr->default_list,
@@ -932,6 +947,7 @@ int snd_use_case_mgr_open(snd_use_case_mgr_t **uc_mgr,
 	INIT_LIST_HEAD(&mgr->value_list);
 	INIT_LIST_HEAD(&mgr->active_modifiers);
 	INIT_LIST_HEAD(&mgr->active_devices);
+	INIT_LIST_HEAD(&mgr->ctl_list);
 	pthread_mutex_init(&mgr->mutex, NULL);
 
 	mgr->card_name = strdup(card_name);
@@ -972,7 +988,7 @@ int snd_use_case_mgr_reload(snd_use_case_mgr_t *uc_mgr)
 	/* reload all use cases */
 	err = import_master_config(uc_mgr);
 	if (err < 0) {
-		uc_error("error: failed to reload use cases\n");
+		uc_error("error: failed to reload use cases");
 		pthread_mutex_unlock(&uc_mgr->mutex);
 		return -EINVAL;
 	}
@@ -1056,7 +1072,6 @@ int snd_use_case_mgr_reset(snd_use_case_mgr_t *uc_mgr)
 /**
  * \brief Get list of verbs in pair verbname+comment
  * \param list Returned list
- * \param verbname For verb (NULL = current)
  * \return Number of list entries if success, otherwise a negative error code
  */
 static int get_verb_list(snd_use_case_mgr_t *uc_mgr, const char **list[])
@@ -1144,8 +1159,10 @@ static int get_supcon_device_list(snd_use_case_mgr_t *uc_mgr,
 
 	modifier = find_modifier(uc_mgr, verb, name, 0);
 	if (modifier) {
-		if (modifier->dev_list.type != type)
+		if (modifier->dev_list.type != type) {
+			*list = NULL;
 			return 0;
+		}
 		return get_list(&modifier->dev_list.list, list,
 				struct dev_list_node, list,
 				name);
@@ -1153,15 +1170,16 @@ static int get_supcon_device_list(snd_use_case_mgr_t *uc_mgr,
 
 	device = find_device(uc_mgr, verb, name, 0);
 	if (device) {
-		if (device->dev_list.type != type)
+		if (device->dev_list.type != type) {
+			*list = NULL;
 			return 0;
+		}
 		return get_list(&device->dev_list.list, list,
 				struct dev_list_node, list,
 				name);
 	}
 
 	return -ENOENT;
-
 }
 
 /**
@@ -1190,41 +1208,201 @@ static int get_conflicting_device_list(snd_use_case_mgr_t *uc_mgr,
 
 #ifndef DOC_HIDDEN
 struct myvalue {
-        struct list_head list;
-        char *value;
+	struct list_head list;
+	const char *text;
 };
 #endif
 
+/**
+ * \brief Convert myvalue list string list
+ * \param list myvalue list
+ * \param res string list
+ * \retval Number of list entries if success, otherwise a negativer error code
+ */
+static int myvalue_to_str_list(struct list_head *list, char ***res)
+{
+	struct list_head *pos;
+	struct myvalue *value;
+	char **p;
+	int cnt;
+
+	cnt = alloc_str_list(list, 1, res);
+	if (cnt < 0)
+		return cnt;
+	p = *res;
+	list_for_each(pos, list) {
+		value = list_entry(pos, struct myvalue, list);
+		*p = strdup(value->text);
+		if (*p == NULL) {
+			snd_use_case_free_list((const char **)p, cnt);
+			return -ENOMEM;
+		}
+		p++;
+	}
+	return cnt;
+}
+
+/**
+ * \brief Free myvalue list
+ * \param list myvalue list
+ */
+static void myvalue_list_free(struct list_head *list)
+{
+	struct list_head *pos, *npos;
+	struct myvalue *value;
+
+	list_for_each_safe(pos, npos, list) {
+		value = list_entry(pos, struct myvalue, list);
+		list_del(&value->list);
+		free(value);
+	}
+}
+
+/**
+ * \brief Merge one value to the myvalue list
+ * \param list The list with values
+ * \param value The value to be merged (without duplicates)
+ * \return 1 if dup, 0 if success, otherwise a negative error code
+ */
+static int merge_value(struct list_head *list, const char *text)
+{
+	struct list_head *pos;
+	struct myvalue *value;
+
+	list_for_each(pos, list) {
+		value = list_entry(pos, struct myvalue, list);
+		if (strcmp(value->text, text) == 0)
+			return 1;
+	}
+	value = malloc(sizeof(*value));
+	if (value == NULL)
+		return -ENOMEM;
+	value->text = text;
+	list_add_tail(&value->list, list);
+	return 0;
+}
+
+/**
+ * \brief Find all values for given identifier
+ * \param list Returned list
+ * \param source Source list with ucm_value structures
+ * \return Zero if success, otherwise a negative error code
+ */
+static int add_identifiers(struct list_head *list,
+			   struct list_head *source)
+{
+	struct ucm_value *v;
+	struct list_head *pos;
+	int err;
+
+	list_for_each(pos, source) {
+		v = list_entry(pos, struct ucm_value, list);
+		err = merge_value(list, v->name);
+		if (err < 0)
+			return err;
+	}
+	return 0;
+}
+
+/**
+ * \brief Find all values for given identifier
+ * \param list Returned list
+ * \param identifier Identifier
+ * \param source Source list with ucm_value structures
+ */
 static int add_values(struct list_head *list,
                       const char *identifier,
                       struct list_head *source)
 {
-        struct ucm_value *v;
-        struct myvalue *val;
-        struct list_head *pos, *pos1;
-        int match;
+	struct ucm_value *v;
+	struct list_head *pos;
+	int err;
         
-        list_for_each(pos, source) {
-                v = list_entry(pos, struct ucm_value, list);
-                if (check_identifier(identifier, v->name)) {
-                        match = 0;
-                        list_for_each(pos1, list) {
-                                val = list_entry(pos1, struct myvalue, list);
-                                if (strcmp(val->value, v->data) == 0) {
-                                        match = 1;
-                                        break;
-                                }
-                        }
-                        if (!match) {
-                                val = malloc(sizeof(struct myvalue));
-                                if (val == NULL)
-                                        return -ENOMEM;
-				val->value = v->data;
-                                list_add_tail(&val->list, list);
-                        }
-                }
-        }
-        return 0;
+	list_for_each(pos, source) {
+		v = list_entry(pos, struct ucm_value, list);
+		if (check_identifier(identifier, v->name)) {
+			err = merge_value(list, v->data);
+			if (err < 0)
+				return err;
+		}
+	}
+	return 0;
+}
+
+/**
+ * \brief compare two identifiers
+ */
+static int identifier_cmp(const void *_a, const void *_b)
+{
+	const char * const *a = _a;
+	const char * const *b = _b;
+	return strcmp(*a, *b);
+}
+
+/**
+ * \brief Get list of available identifiers
+ * \param list Returned list
+ * \param name Name of verb or modifier to query
+ * \return Number of list entries if success, otherwise a negative error code
+ */
+static int get_identifiers_list(snd_use_case_mgr_t *uc_mgr,
+				const char **list[], char *name)
+{
+	struct use_case_verb *verb;
+	struct use_case_modifier *modifier;
+	struct use_case_device *device;
+	struct list_head mylist;
+	struct list_head *value_list;
+	char *str, **res;
+	int err;
+
+	if (!name)
+		return -ENOENT;
+
+	str = strchr(name, '/');
+	if (str) {
+		*str = '\0';
+		verb = find_verb(uc_mgr, str + 1);
+	}
+	else {
+		verb = uc_mgr->active_verb;
+	}
+	if (!verb)
+		return -ENOENT;
+
+	value_list = NULL;
+	modifier = find_modifier(uc_mgr, verb, name, 0);
+	if (modifier) {
+		value_list = &modifier->value_list;
+	} else {
+		device = find_device(uc_mgr, verb, name, 0);
+		if (device)
+			value_list = &device->value_list;
+	}
+	if (value_list == NULL)
+		return -ENOENT;
+
+	INIT_LIST_HEAD(&mylist);
+	err = add_identifiers(&mylist, &uc_mgr->value_list);
+	if (err < 0)
+		goto __fail;
+	err = add_identifiers(&mylist, &verb->value_list);
+	if (err < 0)
+		goto __fail;
+	err = add_identifiers(&mylist, value_list);
+	if (err < 0)
+		goto __fail;
+	err = myvalue_to_str_list(&mylist, &res);
+	if (err > 0)
+		*list = (const char **)res;
+	else if (err == 0)
+		*list = NULL;
+__fail:
+	myvalue_list_free(&mylist);
+	if (err <= 0)
+		return err;
+	qsort(*list, err, sizeof(char *), identifier_cmp);
+	return err;
 }
 
 /**
@@ -1238,8 +1416,7 @@ static int get_value_list(snd_use_case_mgr_t *uc_mgr,
                           const char **list[],
                           char *verbname)
 {
-        struct list_head mylist, *pos, *npos;
-        struct myvalue *val;
+	struct list_head mylist, *pos;
         struct use_case_verb *verb;
         struct use_case_device *dev;
         struct use_case_modifier *mod;
@@ -1272,26 +1449,13 @@ static int get_value_list(snd_use_case_mgr_t *uc_mgr,
                 if (err < 0)
                         goto __fail;
         }
-        err = alloc_str_list(&mylist, 1, &res);
-        if (err >= 0) {
+	err = myvalue_to_str_list(&mylist, &res);
+	if (err > 0)
 	        *list = (const char **)res;
-                list_for_each(pos, &mylist) {
-                        val = list_entry(pos, struct myvalue, list);
-                        *res = strdup(val->value);
-                        if (*res == NULL) {
-                                snd_use_case_free_list((const char **)res, err);
-                                err = -ENOMEM;
-                                goto __fail;
-                        }
-                        res++;
-                }
-        }
+	else if (err == 0)
+		*list = NULL;
       __fail:
-        list_for_each_safe(pos, npos, &mylist) {
-                val = list_entry(pos, struct myvalue, list);
-                list_del(&val->list);
-                free(val);
-        }
+	myvalue_list_free(&mylist);
         return err;
 }
 
@@ -1361,28 +1525,30 @@ int snd_use_case_get_list(snd_use_case_mgr_t *uc_mgr,
                 } else {
                         str = NULL;
                 }
-        	if (check_identifier(identifier, "_devices"))
-          		err = get_device_list(uc_mgr, list, str);
+		if (check_identifier(identifier, "_devices"))
+			err = get_device_list(uc_mgr, list, str);
                 else if (check_identifier(identifier, "_modifiers"))
-                        err = get_modifier_list(uc_mgr, list, str);
-                else if (check_identifier(identifier, "_supporteddevs"))
-                        err = get_supported_device_list(uc_mgr, list, str);
-                else if (check_identifier(identifier, "_conflictingdevs"))
-                        err = get_conflicting_device_list(uc_mgr, list, str);
+			err = get_modifier_list(uc_mgr, list, str);
+		else if (check_identifier(identifier, "_identifiers"))
+			err = get_identifiers_list(uc_mgr, list, str);
+		else if (check_identifier(identifier, "_supporteddevs"))
+			err = get_supported_device_list(uc_mgr, list, str);
+		else if (check_identifier(identifier, "_conflictingdevs"))
+			err = get_conflicting_device_list(uc_mgr, list, str);
 		else if (identifier[0] == '_')
 			err = -ENOENT;
-                else
-                        err = get_value_list(uc_mgr, identifier, list, str);
-        	if (str)
-        		free(str);
-        }
+		else
+			err = get_value_list(uc_mgr, identifier, list, str);
+		if (str)
+			free(str);
+	}
       __end:
 	pthread_mutex_unlock(&uc_mgr->mutex);
 	return err;
 }
 
-static int get_value1(char **value, struct list_head *value_list,
-                      const char *identifier)
+static int get_value1(snd_use_case_mgr_t *uc_mgr, char **value,
+		      struct list_head *value_list, const char *identifier)
 {
         struct ucm_value *val;
         struct list_head *pos;
@@ -1391,18 +1557,22 @@ static int get_value1(char **value, struct list_head *value_list,
 		return -ENOENT;
 
         list_for_each(pos, value_list) {
-              val = list_entry(pos, struct ucm_value, list);
-              if (check_identifier(identifier, val->name)) {
-                      *value = strdup(val->data);
-                      if (*value == NULL)
-                              return -ENOMEM;
-                      return 0;
-              }
+		val = list_entry(pos, struct ucm_value, list);
+		if (check_identifier(identifier, val->name)) {
+			if (uc_mgr->conf_format < 2) {
+				*value = strdup(val->data);
+				if (*value == NULL)
+					return -ENOMEM;
+				return 0;
+			}
+			return uc_mgr_get_substituted_value(uc_mgr, value, val->data);
+		}
         }
         return -ENOENT;
 }
 
-static int get_value3(char **value,
+static int get_value3(snd_use_case_mgr_t *uc_mgr,
+		      char **value,
 		      const char *identifier,
 		      struct list_head *value_list1,
 		      struct list_head *value_list2,
@@ -1410,13 +1580,13 @@ static int get_value3(char **value,
 {
 	int err;
 
-	err = get_value1(value, value_list1, identifier);
+	err = get_value1(uc_mgr, value, value_list1, identifier);
 	if (err >= 0 || err != -ENOENT)
 		return err;
-	err = get_value1(value, value_list2, identifier);
+	err = get_value1(uc_mgr, value, value_list2, identifier);
 	if (err >= 0 || err != -ENOENT)
 		return err;
-	err = get_value1(value, value_list3, identifier);
+	err = get_value1(uc_mgr, value, value_list3, identifier);
 	if (err >= 0 || err != -ENOENT)
 		return err;
 	return -ENOENT;
@@ -1453,7 +1623,7 @@ static int get_value(snd_use_case_mgr_t *uc_mgr,
 				mod = find_modifier(uc_mgr, verb,
 						    mod_dev_name, 0);
 				if (mod) {
-					err = get_value1(value,
+					err = get_value1(uc_mgr, value,
 							 &mod->value_list,
 							 identifier);
 					if (err >= 0 || err != -ENOENT)
@@ -1463,7 +1633,7 @@ static int get_value(snd_use_case_mgr_t *uc_mgr,
 				dev = find_device(uc_mgr, verb,
 						  mod_dev_name, 0);
 				if (dev) {
-					err = get_value1(value,
+					err = get_value1(uc_mgr, value,
 							 &dev->value_list,
 							 identifier);
 					if (err >= 0 || err != -ENOENT)
@@ -1474,7 +1644,7 @@ static int get_value(snd_use_case_mgr_t *uc_mgr,
 					return -ENOENT;
 			}
 
-			err = get_value1(value, &verb->value_list, identifier);
+			err = get_value1(uc_mgr, value, &verb->value_list, identifier);
 			if (err >= 0 || err != -ENOENT)
 				return err;
 		}
@@ -1483,7 +1653,7 @@ static int get_value(snd_use_case_mgr_t *uc_mgr,
 			return -ENOENT;
 	}
 
-	err = get_value1(value, &uc_mgr->value_list, identifier);
+	err = get_value1(uc_mgr, value, &uc_mgr->value_list, identifier);
 	if (err >= 0 || err != -ENOENT)
 		return err;
 
@@ -1530,12 +1700,13 @@ int snd_use_case_get(snd_use_case_mgr_t *uc_mgr,
 	        err = 0;
 	} else if (strcmp(identifier, "_file") == 0) {
 		/* get the conf file name of the opened card */
-		if ((uc_mgr->card_name == NULL)
-		    || (uc_mgr->conf_file_name[0] == '\0')) {
+		if ((uc_mgr->card_name == NULL) ||
+		    (uc_mgr->conf_file_name == NULL) ||
+		    (uc_mgr->conf_file_name[0] == '\0')) {
 			err = -ENOENT;
 			goto __end;
 		}
-		*value = strndup(uc_mgr->conf_file_name, MAX_FILE);
+		*value = strdup(uc_mgr->conf_file_name);
 		if (*value == NULL) {
 			err = -ENOMEM;
 			goto __end;
@@ -1901,3 +2072,54 @@ int snd_use_case_set(snd_use_case_mgr_t *uc_mgr,
 	pthread_mutex_unlock(&uc_mgr->mutex);
         return err;
 }
+
+/**
+ * \brief Parse control element identifier
+ * \param elem_id Element identifier
+ * \param ucm_id Use case identifier
+ * \param value String value to be parsed
+ * \return Zero if success, otherwise a negative error code
+ */
+int snd_use_case_parse_ctl_elem_id(snd_ctl_elem_id_t *dst,
+				   const char *ucm_id,
+				   const char *value)
+{
+	snd_ctl_elem_iface_t iface;
+	int jack_control;
+
+	jack_control = strcmp(ucm_id, "JackControl") == 0;
+	if (!jack_control &&
+	    strcmp(ucm_id, "PlaybackVolume") &&
+	    strcmp(ucm_id, "PlaybackSwitch") &&
+	    strcmp(ucm_id, "CaptureVolume") &&
+	    strcmp(ucm_id, "CaptureSwitch"))
+		return -EINVAL;
+	snd_ctl_elem_id_clear(dst);
+	if (strcasestr(ucm_id, "name="))
+		return __snd_ctl_ascii_elem_id_parse(dst, value, NULL);
+	iface = SND_CTL_ELEM_IFACE_MIXER;
+	if (jack_control)
+		iface = SND_CTL_ELEM_IFACE_CARD;
+	snd_ctl_elem_id_set_interface(dst, iface);
+	snd_ctl_elem_id_set_name(dst, value);
+	return 0;
+}
+
+/**
+ * \brief Parse mixer element identifier
+ * \param dst Simple mixer element identifier
+ * \param ucm_id Use case identifier
+ * \param value String value to be parsed
+ * \return Zero if success, otherwise a negative error code
+ */
+int snd_use_case_parse_selem_id(snd_mixer_selem_id_t *dst,
+				const char *ucm_id,
+				const char *value)
+{
+#ifdef BUILD_MIXER
+	if (strcmp(ucm_id, "PlaybackMixerId") == 0 ||
+	    strcmp(ucm_id, "CaptureMixerId") == 0)
+		return snd_mixer_selem_id_parse(dst, value);
+#endif
+	return -EINVAL;
+}
